/**
 * Copyright (c) 2010 Generative Software Development Lab, University of Waterloo, Canada. All Rights Reserved
 *
 * This library is free software; you can redistribute it and/or modify it under the terms of the 
 * GNU Lesser General Public License as published by the Free Software Foundation; either version 
 * 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
 * the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this library;
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, 
 * MA  02110-1301  USA
 * 
 * You can find the license also here: http://www.gnu.org/copyleft/lesser.html 
 */
package ca.uwaterloo.gsd.icsm2010.wrappers.swing2swt.awt;


import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Widget;

import ca.uwaterloo.gsd.icsm2010.patterns.DelayedInstantiation;
import ca.uwaterloo.gsd.icsm2010.patterns.FactoryMethod;
import ca.uwaterloo.gsd.icsm2010.patterns.WrappingIdentityMap;

/**
 * This type implements the composite role of a composite pattern and
 * maps thus to an SWT Composite. It also implements the main triggering
 * for delayed instantiation since we can instantiate adaptees when they
 * are added to a container.
 * 
 * @author Thiago Tonelli Bartolomei
 */
public class Container extends Component {

	@FactoryMethod
	protected void createAdaptee(Composite parent) {
		this.setAdaptee(new Composite(parent, SWT.NONE));
	}
	
	@WrappingIdentityMap
	protected Container() {}
	
	@WrappingIdentityMap
	Container(Widget w) {
		this.setAdaptee(w);
	}
	
	@DelayedInstantiation
	private List<Component> pending = null;
	
	@DelayedInstantiation
	protected void addPending(Component c) {
		getPending().add(c);
	}
	
	@DelayedInstantiation
	List<Component> getPending() {
		if (null == pending)
			pending = new ArrayList<Component>();
		return pending;
	}
	
	@DelayedInstantiation
	protected void createPending() {
		if (null != pending && ! pending.isEmpty()) {
			for (Component c : pending) {
				this.wire(c);
			}
			pending = null;
		}
	}
	
	@DelayedInstantiation
	protected void wire(Component c) {
		
		// Note that this cast will fail if the container is not a composite (like JButton -> Button)
		c.createAdaptee((Composite) this.getAdaptee());
		if (c instanceof Container) 
			((Container) c).createPending();
	}
	
	// Public API
	public Component add(Component comp) {
		
		if(null == getAdaptee()) {
			this.addPending(comp);
		} else {
			this.wire(comp);
		}
		return comp;
	}

	public void setLayout(LayoutManager mgr) {
		// Currently we just support flow layout. We would have to change LayoutManager's
		// public API to make this transparent.
		((Composite) getAdaptee()).setLayout(((FlowLayout) mgr).getAdaptee());
	}

	@WrappingIdentityMap
	public Component[] getComponents() {
		// Inheritance Hierarchy mismatch. Containers can map to parents of Composite.
		if (! (getAdaptee() instanceof Composite)) {
			return new Component[0];
		}

		Control[] controls = ((Composite) getAdaptee()).getChildren();
		Component[] components = new Component[controls.length];
		for(int i = 0; i < controls.length; i++) {
			components[i] = getSurrogate(controls[i], Component.class); 
		}
		return components;
	}
}
