/**
 * Copyright (c) 2010 Generative Software Development Lab, University of Waterloo, Canada. All Rights Reserved
 *
 * This library is free software; you can redistribute it and/or modify it under the terms of the 
 * GNU Lesser General Public License as published by the Free Software Foundation; either version 
 * 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
 * the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this library;
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, 
 * MA  02110-1301  USA
 * 
 * You can find the license also here: http://www.gnu.org/copyleft/lesser.html
 */
package ca.uwaterloo.gsd.icsm2010.wrappers.swt2swing.widgets.internal;

import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;

import ca.uwaterloo.gsd.icsm2010.patterns.LayeredAdapter;
import ca.uwaterloo.gsd.icsm2010.wrappers.swt2swing.SWT;

/**
 * Implements the Layered Adapter Pattern for Button.
 * 
 * The IButton interface represents the contract expected from Button
 * counterparts in Swing. Here we implement the contract as a JButton
 * or a JCheckBox, and provide a factory to create the appropriate 
 * object depending on the runtime style constant.
 * 
 * @author Thiago Tonelli Bartolomei
 */
@LayeredAdapter
public interface IButton {
	
	/**
	 * A factory to create the appropriate variant of IButton.
	 * 
	 * @author Thiago Tonelli Bartolomei
	 */
	public static class IButtonFactory {
		
		public static IButton createIButton(int style) {
			if ((style & SWT.PUSH) != 0)
				return new JButtonAdapter();

			else if ((style & SWT.CHECK) != 0)
				return new JCheckBoxAdapter();
			
			return null;
		}
	}
	
	// Public API expected by Button's surrogate
	public void setText(String text);
	
	public void addActionListener(ActionListener listener);
	
	public boolean isSelected();
}

/**
 * An IButton variant based on JButton.
 * 
 * We use class adapters because we can take advantage of the fact that
 * IButton declares methods that are already implemented by JButton.
 * 
 * @author Thiago Tonelli Bartolomei
 */
@SuppressWarnings("serial")
class JButtonAdapter extends JButton implements IButton {
}

/**
 * An IButton variant based on JCheckBox.
 * 
 * @author Thiago Tonelli Bartolomei
 */
@SuppressWarnings("serial")
class JCheckBoxAdapter extends JCheckBox implements IButton {
}
