/**
 * Copyright (c) 2010 Generative Software Development Lab, University of Waterloo, Canada. All Rights Reserved
 *
 * This library is free software; you can redistribute it and/or modify it under the terms of the 
 * GNU Lesser General Public License as published by the Free Software Foundation; either version 
 * 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See 
 * the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this library;
 * if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, 
 * MA  02110-1301  USA
 * 
 * You can find the license also here: http://www.gnu.org/copyleft/lesser.html
 */
package ca.uwaterloo.gsd.icsm2010.wrappers.swt2swing.widgets.internal;

import java.awt.Dimension;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JScrollPane;

import ca.uwaterloo.gsd.icsm2010.patterns.ClassAdapter;
import ca.uwaterloo.gsd.icsm2010.patterns.LayeredAdapter;
import ca.uwaterloo.gsd.icsm2010.patterns.ObjectAdapter;
import ca.uwaterloo.gsd.icsm2010.wrappers.swt2swing.SWT;

/**
 * Implements the Layered Adapter Pattern for List.
 * 
 * The IList interface represents the contract expected from List
 * counterparts in Swing. Here we implement the contract as a JList
 * or as a JList inside a JScrollPane, and provide a factory to create 
 * the appropriate object depending on the runtime style constant.
 * 
 * 
 * @author Thiago Tonelli Bartolomei
 */
@LayeredAdapter
public interface IList {

	/**
	 * A factory to create the appropriate variant of IList.
	 * 
	 * @author Thiago Tonelli Bartolomei
	 */
	public static class IListFactory {
		
		public static IList createIList(int style) {
			if ((style & SWT.V_SCROLL) != 0)
				return new ScrollableJListAdapter();
			else 
				return new SimpleJListAdapter();
			}
	}
	
	public boolean isSelectedIndex(int index);
	
	public void addElement(String s);
	
	public int getItemCount(); // using source API name because of name clash
	
	public String get(int index);
	
	public void remove(int index);
	
	public void add(int index, String item);
}

/**
 * An IList variant based on a simple JList with its model.
 * 
 * This variant combines a class adapter to JList with an object adapter
 * to DefaulListModel. The methods of IList that are implemented by JList
 * can be left to the super-type, but the methods that access the model
 * need to be re-implemented.
 * 
 * @author Thiago Tonelli Bartolomei
 */
@SuppressWarnings("serial")
@ClassAdapter
class SimpleJListAdapter extends JList implements IList {
	
	@ObjectAdapter
	DefaultListModel model;
	
	SimpleJListAdapter() {
		super();
		model = new DefaultListModel();
		this.setModel(model);
	}
	
	public void addElement(String s) {
		model.addElement(s);
	}
	
	public int getItemCount() {
		return model.getSize();
	}
	
	public String get(int index) {
		// Type Mismatch
		return (String) model.get(index);
	}
	
	public void remove(int index) {
		model.remove(index);
	}
	
	public void add(int index, String item) {
		model.add(index, item);
	}
}

/**
 * An IList variant based on a JList inside a JScrollPane.
 * 
 * This variant is used if the List is configured to be scrollable. It uses
 * a class adapter to JScrollPane and an object adapter to the SimpleJListAdapter
 * defined above. This way, all methods simply delegate to the simple JList adapter
 * but the GUI widget presents a scroll pane.
 * 
 * @author Thiago Tonelli Bartolomei
 */
@SuppressWarnings("serial")
@ClassAdapter
class ScrollableJListAdapter extends JScrollPane implements IList {

	@ObjectAdapter
	IList list;
	
	ScrollableJListAdapter() {
		super();
		SimpleJListAdapter l = new SimpleJListAdapter();
		getViewport().setView(l);
		setPreferredSize(new Dimension(100, 70));
		list = l;
	}
	
	public boolean isSelectedIndex(int index) {
		return list.isSelectedIndex(index);
	}
	
	public void addElement(String s) {
		list.addElement(s);
	}
	
	public int getItemCount() {
		return list.getItemCount();
	}
	
	public String get(int index) {
		return list.get(index);
	}
	
	public void remove(int index) {
		list.remove(index);
	}
	
	public void add(int index, String item) {
		list.add(index, item);
	}
}